# mdast(3) -- Markdown processor

## SYNOPSIS

```javascript
// Load dependencies:
var mdast = require('./index.js');
var html = require('mdast-html');
var yamlConfig = require('mdast-yaml-config');

// Use plugins:
var processor = mdast().use(yamlConfig).use(html);

// Process the document:
var doc = processor.process([
    '---',
    'mdast:',
    '  commonmark: true',
    '---',
    '',
    '2) Some *emphasis*, **strongness**, and `code`.'
].join('\n'));
```

## DESCRIPTION

This is the application programming interface documentation for **mdast**.
To find documentation for the command line interface, see **mdast**(1).

## mdast.use(plugin\[, options\])

Change the way **mdast** works by using a plugin.  Plugins are documented
at <https://github.com/wooorm/mdast/blob/master/doc/plugins.md>.

**Signatures**

*   `processor = mdast.use(plugin[, options])`;
*   `processor = mdast.use(plugins)`.

**Parameters**

*   `plugin` (`Function`) — Plugin.
*   `plugins` (`Array.<Function>`) — List of plugins.
*   `options` (`Object?`) — Passed to plugin.  Specified by its documentation.

**Returns**

`Object` — An instance of **mdast**.  The instance functions just like the
**mdast** library itself (it has the same methods), but caches the `use`d
plugins.

## mdast.parse(file\[, options\])

Parse a markdown document into an **mdastnode**(7).

**Signatures**

*   `node = mdast.parse(file|value[, options])`.

**Parameters**

*   `file` (`VFile`) — Virtual file;
*   `value` (`string`) — String representation of a file;
*   `options` (`Object`) — Configuration given to the parser.

**Returns**

`Node` — Node.  Nodes are documented at **mdastnode**(7).

## mdast.run(node\[, file\]\[, done\])

Transform a node by applying plug-ins to it. Either a node or a file which
was previously passed to `parse()`, must be given.

**Signatures**

*   `node = mdast.run(node[, file|value][, done])`;
*   `node = mdast.run(file[, done])`.

**Parameters**

*   `node` (`Object`) — Node as returned by `parse()`, see **mdastnode**(7);

*   `file` (`VFile`) — Virtual file;

*   `value` (`string`) — String representation of a file;

*   `done` (`function done(err, node, file)`) — See
    FUNCTION DONE(ERR, NODE, FILE).

**Returns**

`Node` — The given node.

**Throws**

When no `node` was given and no node was found on the file.

### function done(err, node, file)

Invoked when transformation is complete.

**Signatures**

*   `function done(err)`;
*   `function done(null, node, file)`.

**Parameters**

*   `err` (`Error`) — Failure;
*   `node` (`Node`) — Transformed node;
*   `file` (`File`) — File object representing the input file;

## mdast.stringify(node\[, file\]\[, options\])

Compile a node into a document.

**Signatures**

*   `doc = mdast.stringify(node[, file|value][, options])`;
*   `doc = mdast.stringify(file[, options])`.

**Parameters**

*   `node` (`Object`) — Node as returned by `parse()`, see **mdastnode**(7);
*   `file` (`VFile`) — Virtual file;
*   `value` (`string`) — String representation of a file;
*   `options` (`Object`) — Configuration.

**Returns**

`doc` (`string`) — Document.

**Throws**

When no `node` was given and no node was found on the file.

## mdast.process(file\[, options\]\[, done\])

Parse, transform, and compile markdown into something else.

**Signatures**

*   `doc? = mdast.process(file|value[, options][, done])`.

**Parameters**

*   `file` (`File`) — Virtual file;
*   `value` (`string`) — Source of a (virtual) file;
*   `options` (`Object`) — Settings.  See **mdastsetting**(7);
*   `done` (`function done(err?, doc?, file?)`.

**Returns**

`string?` — Document.  Formatted in markdown by default, or in whatever a
plugin generates. When an async transformer is used, `null` is returned and
`done` must be given to receive the results upon completion.

### function done(err?, doc?, file?)

Invoked when processing is complete.

**Signatures**

*   `function done(err)`;
*   `function done(null, doc, file)`.

**Parameters**

*   `err` (`Error`) — Failure;
*   `doc` (`string`) — Document generated by the process;
*   `file` (`File`) — File object representing the input file;

## FileSet()

**mdast**(1) compiles multiple files using a `FileSet` instance.  This set
is exposed to plug-ins as an argument to the attacher. `FileSet`s
should not be created by plug-ins.

## fileSet.valueOf()

## fileSet.toJSON()

Get access to the file objects in a set.

**Signatures**

*   `files = fileSet.valueOf()`.

**Returns**

`Array.<File>` — List of files being processed by **mdast**(1).

## fileSet.use(completer)

Add a completer to the middleware pipeline of a file-set.  When all
files are transformed, this pipeline is run and `completer` is invoked
with `fileSet`.

**Signatures**

*   `fileSet.use(completer)`.

**Parameters**

*   `completer` (`Function`).

## fileSet.add(file|filePath)

Add a new file to be processed by **mdast**(1). The given file is
processed just like other files, with a few differences.

Programmatically added files are:

*   Ignored when their file-path is already added;
*   Never written to the file-system;
*   Not logged about.

**Signatures**

*   `fileSet.use(filePath)`;
*   `fileSet.use(file)`.

**Parameters**

*   `filePath` (`string`) - Path to virtual file;
*   `file` (`File`) - Virtual file.

## BUGS

<https://github.com/wooorm/mdast/issues>

## SEE ALSO

**mdast**(1), **mdastconfig**(7), **mdastnode**(7), **mdastsetting**(7).

## AUTHOR

Written by Titus Wormer <mailto:tituswormer@gmail.com>
